
// ***************** RK NOG'S DMF FLOPPY FORMAT PROGRAM *****************
//						   		1.745408 MEGABYTES
//							82 TRACKS @ 21 SECTORS PER TRACK
//
//											Comments
//
//	A.     THIS PROGRAM USES ONLY ONE HEADER FILE.
//
// B.     LINES WITH //***** DISRUPT ES:BX.  IF YOU WANT TO UNDERSTAND
// 	    THE PROCEDURE, YOU CAN RUN THE PROGRAM WITH //***** REMOVED,
//			 BUT THE DMF FORMAT WILL NOT WORK PROPERLY, UNLESS YOU KNOW
//			 HOW TO PUSH AND POP THE RIGHT REGISTERS.
//
// C.     YOU MUST BE IN MS_DOS MODE TO RUN THIS PROGRAM, UNLESS YOU
//			 KNOW HOW YOU CAN CATCH THE MS_DOS PRIVELEGE LEVEL FROM WINDOWS.
//
// D.     BETTOR SECTOR GAPS, DELAY TIMES, AND INTERLEAVE MIGHT BE USED.
//			 I USED PRIMITIVE DELAY FUNCTIONS WITH DELAY TIMES SET FOR
//			 MODEST PENTIUMS.
//
//	E.		 IT IS RECOMMENDED THAT YOU FORMAT ONLY FRESHLY FORMATTED
//			 1.457664 MEGABYTE DISKS TO PREVENT CALLIBRATE AND SEEK ERRORS.
//			 THAT THERE ARE NO COUNTER MEASURES TO THESE ERRORS, IS A
//			 WEAKNESS IN THIS PROGRAM.
//
// F.     CONSTRUCTIVE COMMENTS ARE WELCOME.
//
//
// 										Bibliography
//
// 1.     Help with DMA programming is found in The Indespensable PC Hardware
// 		 Book, Messmer, Addison-Wesley Longman Limited, 1997, page 721.
//
// 2.     Help with port programming is found in The Undocumented PC,
// 		 Gilluwe, Addison-Wesley Developers Press, 1997, pages 525, 1057.
//
//**************************************************************************

#include <stdio.h> 	// for fprintf, printf, getc

// ********************************** PROTOTYPES ***************************
 int delay(void);
 int turn_drive_motor_off(void);
 int turn_drive_motor_on(int on_off_switch);
 int position_cursor(int row, int collumn);
 int set_color(int roak_color);
 int send_controller(unsigned char command_byte);
 int receive_controller(void);
 int setup_dma_controller(int dma_command, int low_part_buffer_size, int high_part_buffer_size);
 int recalibrate_a(void);
 int read_sector_id(int side);
 int seek_track_side(int track, int side);
 int read_sector(int sector_number);

// ******************************* DECLARE VARIABLES ***********************

	int counter, d_counter, s_counter, r_counter;

	unsigned char receive_array[7], format_buffer[84], track_buffer[6], wfs[512];

	unsigned char sector_0[512] = {235,64,144,// 0EBh = 235d = short jump
			40,121,78,73,61,73,72,67,			// OEM
			// START OF BIOS PARAMETER BLOCK
			0,2,												// 2 * 256 = 512 bytes per sector
			1, 											// sectors per cluster
			1,0,											// # sectors before FAT
			2,												// FAT count
			224,0,										// max # entries in root dir
			116, 13,										// total sectors
			240,											// media
			10,0,											// FAT size
			21,0,											// sectors per track
			2,0,											// head count
			0,0,0,0,										// hidden sectors
			208,2,0,0,									// huge sectors
			// END OF BIOS PARAMETER BLOCK
			0, 											// drive number
			0,												// reserved
			41,											// boot sign = )
			190,186,237,254,							// serial number =  FEED BABE   FACE-ABED = ED AB CE FA = 237,171,206,250,
			82,153,142,75,32,78,79,71,32,32,32,	// volume label RK NOG___
			70,65,84,49,50,32,32,32,0,2,		 	// FAT12
			// PROGRAM CODE STARTS AT OFSET 1Eh = 30d = 0001 1110b
			112,0,250,51,201,142,209,188,252,123,22,7,189,120,0,197,118,0,30,86,22,85,
			96,190,136,125,232,246,0,97,191,34,5,137,126,0,137,78,2,177,11,252,243,164,
			6,31,189,0,124,198,69,254,15,139,70,24,136,69,249,251,51,201,138,70,16,152,
			247,102,22,3,70,14,19,209,139,118,17,96,137,70,252,137,86,254,184,32,0,247,
			230,139,94,11,3,195,72,247,243,1,70,252,17,78,254,97,232,79,0,114,31,191,0,
			7,128,61,0,116,23,96,177,11,190,112,125,243,166,97,116,28,78,116,9,131,199,
			32,59,251,114,230,235,220,190,185,125,232,129,0,152,205,22,94,31,102,143,4,
			205,25,139,125,26,141,69,254,138,78,13,247,225,3,70,252,19,86,254,177,4,232,
			9,0,114,216,234,0,2,112,0,3,65,187,0,7,80,176,3,136,134,249,0,88,96,80,139,
			70,24,247,102,26,145,88,51,210,247,241,146,139,78,24,246,241,42,204,145,254,
			197,134,233,10,241,134,242,135,202,184,1,2,178,0,14,7,205,19,115,24,51,192,
			51,210,205,19,97,138,150,249,0,254,202,136,150,249,0,128,250,0,117,190,249,
			195,97,64,128,199,2,73,117,172,195,30,14,31,172,60,0,116,9,180,14,187,7,0,205,16,235,242,31,195,0,0,0,0,0,0,0,0,0,0,
			// I O _ _ _ _ _ _ S Y S M S D O S _ _ _
			73,79,32,32,32,32,32,32,83,89,83,77,83,68,79,83,32,32,32,
			// S Y S _ _ B _ o _ o _ t _ i _ n _ g _ _ S _ y _ s _ t _ e _ m _ . _ . _  .
			83,89,83,0,0,66,32,111,32,111,32,116,32,105,32,110,32,103,32,32,83,32,121,32,115,32,116,32,101,32,109,32,46,32,46,32,46,
			13,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			// Non-system_disk_or_boot_error,
			78,111,110,45,115,121,115,116,101,109,32,100,105,115,107,32,111,114,32,98,111,111,116,32,101,114,114,111,114,44,
			// 13d = 0Dh = 0000 1101b   10d = 0Ah = 0000 1010b
			//carriage return & line feed & line feed
			13,10,10,
			// Replace_disk_and_re-boot.
			82,101,112,108,97,99,101,32,100,105,115,107,32,97,110,100,32,114,101,45,98,111,111,116,46,
			// 13d = 0Dh = 0000 1101b   10d = 0Ah = 0000 1010b
			//carriage return & line feed
			13,10,
			// 55h = 85d = 0101 0101b  0AAh = 170d = 1010 1010b
			//	representation for end of boot sector
			0,0,0,0,0,0,0,0,0,85,170};

	unsigned char sector_1_11[512]= {240,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

	unsigned char sector_21[512] = {/*root dir RK NOG*/82,153,142,75,32,78,79,71,/*file extension*/32,32,32,/*file attribute*/40,/*reserved*/0,0,0,0,0,0,0,0,0,0,/*time*/0,0,/*date*/0,0,/*cluster number*/0,0,/*file size*/0,0,0,0,/*end root dir*/0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};


// ********************************** FUNCTIONS ****************************


// DELAY
int delay(){
	__asm{
		mov AH, 86h         		// system wait function
		mov CX, 00h         		// CX = 1E DX = 8480h for 2 seconds
		mov DX, 5000h				// 5000h = 20480d =
		int 15h             		// 20.480 milliseconds
		}return 0;}


// TURN FLOPPY MOTOR OFF
int turn_drive_motor_off(){
	__asm{
	mov DX, 3F2h
	mov AL, 0h
	OUT DX, AL
	}return 0;}


// TURN FLOPPY MOTOR ON
int turn_drive_motor_on(int on_off_switch){
	__asm{          // enable motor B: on_off_switch = 45d = 2Dh = 0010 1101b
	mov DX, 3F2h 	 // enable motor A: on_off_switch = 28d = 1Ch = 0001 1100b
	mov AL, BYTE PTR on_off_switch
	OUT DX, AL
	}return 0;}


// POSITION CURSOR
int position_cursor(int row, int collumn){
	__asm{
	mov AH, 02h						// function number
	mov BH, 00h						// video page
	mov DH, BYTE PTR row			// row
	mov DL, BYTE PTR collumn	// collumn
	int 10h							// interrupt call
	}return 0;}


// IMPROVE VIDEO AND CHANGE BACKGROUND AND FONT COLOR
int set_color(int roak_color){
	__asm{
	mov AH, 09h						// function number
	mov CX, 2000					// number of characters of output
	mov BL, BYTE PTR roak_color// color code
	mov AL, " "						// blank characters
	int 10h							// call interrupt
	}return 0;}


// SEND COMMAND BYTES TO FLOPPY CONTROLLER
int send_controller(unsigned char command_byte){
	int mrq = 0, compare = 0;
	for(;;){             		 // START FOR (;;)
	__asm{               		 // START ASM
			mov DX, 3F4h
			IN AL, DX
			mov BYTE PTR compare, AL
			}                 	 // END ASM
	mrq = compare & 0x0080; 	 // Main ReQuest data status register
	if(mrq == 0x0080){          // is ok when 128d = 80h = 1000 0000b
	delay();
		__asm{               	 // START ASM
			mov DX, 3F5h
			mov AL, BYTE PTR command_byte
			OUT DX, AL
			}                 	 // END ASM
	return 0;}}}


// RECEIVE RESULT BYTES FROM FLOPPY CONTROLLER
int receive_controller(){
	int mrq = 0, compare = 0;
	unsigned char result_byte;
	for(;;){              		  // START FOR (;;)
	__asm{             			  // START ASM
			mov DX, 3F4h
			IN AL, DX
			mov BYTE PTR compare, AL
			}               		  // END ASM
	mrq=compare & 0x0080;  		  // Main ReQuest data status register
	if(mrq==0x0080){             // when 128d = 80h = 1000 0000b
	delay();
	__asm{               		  // START ASM
			mov DX, 3F5h
			IN AL, DX
			mov BYTE PTR result_byte, AL
			}                      // END ASM
	return result_byte;}}}


// PROGRAMMING OF 8237A DMA CONTROLLER
// SETUP DMA FOR READ
//	setup_dma_controller(0x0056, 0x00FF, 0x0001);
// SETUP DMA FOR WRITE
//	setup_dma_controller(0x005A, 0x00FF, 0x0001);
// SETUP DMA FOR DMF FORMAT
//	setup_dma_controller(0x005A, 0x0053, 0x0000);
// SETUP DMA FOR REGULAR FORMAT
//	setup_dma_controller(0x005A, 0x0047, 0x0000);
int setup_dma_controller(int dma_command, int low_part_buffer_size, int high_part_buffer_size){
	__asm{
// DISABLE INTERRUPTS
	cli

// DISABLE DMA1
	MOV AL, 14h
	OUT 08h, AL       	// DMA-1 Command Register port 8h
								// command 20d = 14h = 0001 0100b
								// rotating priority; disable
// MODE
	MOV AL, BYTE PTR dma_command
		// dma_command = 90d = 5Ah = 0101 1010b write sector or format track
		// dma_command = 86d = 56h = 0101 0110b for read sector
	OUT 0Bh, AL				// DMA-1 Mode Register port 0Bh
								// single mode; auto; write; channel 2
// SPLIT
	MOV AX, ES
	MOV CL, 04h
	SHL AX, CL
	ADD AX, BX
	JC carry

// NO CARRY
	MOV BX, ES
	MOV CL, 04h
	SHR BH, CL
	JMP buffer_address
carry:
	MOV BX, ES
	MOV CL, 04h
	SHR BH, CL
	ADC BH, 00h
buffer_address:
	OUT 0Ch, AL                		// DMA-1 Clear Byte Flip-Flop
	OUT 04h, AL							  	// reset flip-flop
												// DMA-1 Channel 2 Output
												// Low Order Address Byte
	MOV AL, AH
	OUT 04h, AL                     	// DMA-1 Channel 2 Output
												// High Order Address Byte
	MOV AL, BH                       // page address
	OUT 81h, AL                      // load page register

// COUNT VALUE
	OUT 0Ch, AL                      // DMA-1 Clear Byte Flip-Flop
												// reset flip-flop
// BUFFER SIZE LOW PART
	MOV AL, BYTE PTR low_part_buffer_size
			// buffer size = 83d = 53h = 0101 0011b for 84 bytes DMF format
			// buffer size = 71d = 47h = 0100 0111b for 72 bytes 1.44 Mb format
			// buffer size = 255d = 0FFh = 1111 1111b for read or write sector
	OUT 05h, AL							  	// DMA-1 Count Port
												// load low order count byte
// BUFFER SIZE HIGH PART
	MOV AL, BYTE PTR high_part_buffer_size
			// buffer size = zero for format
			// buffer size = 1 = 256d + 155d = 511d for 512 bytes read or write
	OUT 05h, AL                      // DMA-1 Count Port
												// load high order count byte

// UNMASK CHANNEL
	MOV AL, 02h
	OUT 0Ah, AL                  		// DMA-1 Mask Register Bit
												// command 2d = 2h = 0000 0010b
												// clear mask channel 2
// ENABLE DMA1
	MOV AL, 10h
	OUT 08h, AL                      // DMA-1 Command Register Port
												// 10d = 0Ah = 0001 0000b
// ENABLE INTERRUPTS
	sti}return 0;}


// RECALLIBRATE DRIVE A:
int recalibrate_a(){
	send_controller(0x0007);   // command byte
	send_controller(0x0000);	// DRIVE A:
// SENSE INTERRUPT STATUS
	send_controller(0x0008);
	receive_array[0] = receive_controller();
	if((receive_array[0] & 0x0080) == 0x0080)return 1;
	receive_array[1] = receive_controller();
//*****	printf("\n     Recalibrate Code Is %d, For Track %d.     ", receive_array[0], receive_array[1]);
	return 0;}


// READ SECTOR IDENTIFICATION
int read_sector_id(int side){
	send_controller(0x004A);	// 74d = 4Ah = 0100 1010b
	send_controller(side*4);
	for(counter = 0; counter < 7; counter++){
	receive_array[counter] = receive_controller();
//*****	position_cursor(counter + 7, 0);
//*****	printf("\n     Side Result Byte %d = %d.     ", receive_array[counter]);
	}
	return 0;}


// SEEK TRACK AND SIDE
int seek_track_side(int track, int side){
do_it_again:;
	send_controller(0x000F);
	send_controller(side*4);
	send_controller(track);
// SENSE INTERRUPT STATUS
	send_controller(0x0008);
	receive_array[2] = receive_controller();
	if((receive_array[2] & 0x0080) == 0x0080){
			delay();
//*****			printf("\n     Seeking Track %d.", track);
			goto do_it_again;}
	receive_array[3] = receive_controller();
//*****	printf("\n     Seek Code Is %d, For Track %d.     ", receive_array[2], receive_array[3]);
	return 0;}


// GET MEDIA ID BY READING BOOT SECTOR
int read_sector(int sector_number){
	unsigned char buffy[512];
   buffy[21] = 0;
	__asm{
		  mov AL, 00h    								// drive
		  mov CX, 01h
		  mov DX, WORD PTR sector_number
		  LEA BX, WORD PTR buffy
		  int 25h
		  }
	return buffy[21];
}


// ********************************** MAIN PROGRAM *************************


void main(){
// DECLARE VARIABLES
	int track, side, t, s, MI, frc, delay_fmt, delay_write, sector;
    char huh;
 
// READ BOOT SECTOR
	MI = 0;
	MI = read_sector(0);
	printf("\n\n     YOU MUST BE IN MS_DOS MODE TO RUN THIS PROGRAM!\n     THE MEDIA ID OF THE DISK IN DRIVE A: = %d.\a\a\a", MI);
	if(MI != 240)printf("\n\n     Make sure there is a freshly formatted 1.457664\n     megabyte floppy disk in drive A!\a\a\a");

// INITIAL STATEMENT
	printf("\n\n     If you want to format the 1.457664 megabyte floppy disk\n     in drive A: to 1.745408 megabytes, enter 'Y' or 'y': \a\a\a");
	huh = getc(stdin);
	if (huh != 'Y' && huh != 'y'){
        printf("\n\n     You chose not to format a floppy disk using ports.\n\n     Freehafer's DMF Floppy Format Program Is Terminated.\n     (c)2002 INNIEA PUBLISHING COMPANY  R%c%cK NOG!\n\n",(char)153,(char)142);
		__asm{
			mov AX, 4C00h
			int 21h}}

// DELAY BEFORE FORMAT
	for(delay_fmt = 0; delay_fmt < 15; delay_fmt++){
	__asm{
		mov AH, 86h         		// system wait function
		mov CX, 02h         		// CX = 1E DX = 8480h for 2 seconds
		mov DX, 00h					// 131.072 milliseconds
		int 15h             		//
		}
	position_cursor(0,0);
	set_color(0x003F);
	printf("\n\n     TINY BUBBLES MAKE ME FEEL FINE!\n\n     THIS IS A DELAY BEFORE DMF FORMAT.  \a  ");}

// TURN ON A: DRIVE MOTOR
	turn_drive_motor_on(0x001C);
	delay();
					  
// IMPROVE SCREEN
	position_cursor(0,0);
	set_color(0x005F);

// RECALIBRATE A:
	for(frc = 0; frc < 5; frc++){
	recalibrate_a();}

// START OF FORMAT LOOPS
	printf("\n\n\n\n\n\n\n\n\n");
	for(t = 0; t < 82; t++){
	for(s = 0; s < 2; s++){
	track = t; side = s;

// FIX VARIABLES
	format_buffer[ 0] = track; format_buffer[ 1] = side; format_buffer[ 2] =  1; format_buffer[ 3] = 2;	track = t; side = s;
	format_buffer[ 4] = track; format_buffer[ 5] = side; format_buffer[ 6] = 12; format_buffer[ 7] = 2;	track = t; side = s;
	format_buffer[ 8] = track; format_buffer[ 9] = side; format_buffer[10] =  2; format_buffer[11] = 2;	track = t; side = s;
	format_buffer[12] = track; format_buffer[13] = side; format_buffer[14] = 13; format_buffer[15] = 2;	track = t; side = s;
	format_buffer[16] = track; format_buffer[17] = side; format_buffer[18] =  3; format_buffer[19] = 2;	track = t; side = s;
	format_buffer[20] = track; format_buffer[21] = side; format_buffer[22] = 14; format_buffer[23] = 2;	track = t; side = s;
	format_buffer[24] = track; format_buffer[25] = side; format_buffer[26] =  4; format_buffer[27] = 2;	track = t; side = s;
	format_buffer[28] = track; format_buffer[29] = side; format_buffer[30] = 15; format_buffer[31] = 2; 	track = t; side = s;
	format_buffer[32] = track; format_buffer[33] = side; format_buffer[34] =  5; format_buffer[35] = 2;	track = t; side = s;
	format_buffer[36] = track; format_buffer[37] = side; format_buffer[38] = 16; format_buffer[39] = 2; 	track = t; side = s;
	format_buffer[40] = track; format_buffer[41] = side; format_buffer[42] =  6; format_buffer[43] = 2;	track = t; side = s;
	format_buffer[44] = track; format_buffer[45] = side; format_buffer[46] = 17; format_buffer[47] = 2;  	track = t; side = s;
	format_buffer[48] = track; format_buffer[49] = side; format_buffer[50] =  7; format_buffer[51] = 2; 	track = t; side = s;
	format_buffer[52] = track; format_buffer[53] = side; format_buffer[54] = 18; format_buffer[55] = 2; 	track = t; side = s;
	format_buffer[56] = track; format_buffer[57] = side; format_buffer[58] =  8; format_buffer[59] = 2; 	track = t; side = s;
	format_buffer[60] = track; format_buffer[61] = side; format_buffer[62] = 19; format_buffer[63] = 2;	track = t; side = s;
	format_buffer[64] = track; format_buffer[65] = side; format_buffer[66] =  9; format_buffer[67] = 2; 	track = t; side = s;
	format_buffer[68] = track; format_buffer[69] = side; format_buffer[70] = 20; format_buffer[71] = 2;	track = t; side = s;
	format_buffer[72] = track; format_buffer[73] = side; format_buffer[74] = 10; format_buffer[75] = 2; 	track = t; side = s;
	format_buffer[76] = track; format_buffer[77] = side; format_buffer[78] = 21; format_buffer[79] = 2; 	track = t; side = s;
	format_buffer[80] = track; format_buffer[81] = side; format_buffer[82] = 11; format_buffer[83] = 2;	track = t; side = s;
	track_buffer[0] = 0x004D; track_buffer[1] = side*4; track_buffer[2] = 0x0002;    side = s;
	track_buffer[3] = 0x0015; track_buffer[4] = 0x000C; track_buffer[5] = 0x0000; // fill byte

// POSITION SCREEN
//*****	position_cursor(0,0);

// VERIFY CYLINDER
	seek_track_side(track, side);
	track = t;
	side = s;

// VERIFY HEAD
	read_sector_id(side);
	side = s;

// PUT BUFFER INTO ES:BX
	__asm{LEA BX, format_buffer}

// SETUP DMA FOR DMF FORMAT
	setup_dma_controller(0x005A, 0x0053, 0x0000);

// SEND COMMAND BYTES TO CONTROLLER
	for(s_counter = 0; s_counter < 6; s_counter++){
	send_controller(track_buffer[s_counter]);}

// RECEIVE RESULT BYTES FROM CONTROLLER
	for(r_counter = 0; r_counter < 7; r_counter++){
	receive_array[r_counter] = receive_controller();
//*****	position_cursor(r_counter + 7, 30);
//*****	printf("\n     Format Result Byte %d = %d.     ", r_counter, receive_array[r_counter]);
	}

// END OF FORMAT LOOPS
	fprintf(stderr,"     Formatting . . . Track %d . . . Side %d . . .     \r ", track, side);}}
//	printf("\n     Formatting . . . Track %d . . . Side %d . . .     ", track, side);}}

// TURN OFF FLOPPY A: MOTOR
	turn_drive_motor_off();

// DELAY BEFORE WRITE
	for(delay_write = 0; delay_write < 7; delay_write++){
	position_cursor(0,0);
	set_color(0x006D);
	printf("\n\n     TINY BUBBLES DO IT EVERY TIME!\n\n     THIS IS A DELAY BEFORE FAT SECTORS ARE WRITTEN. \a ");}
                    
// WRITE SECTOR 0
	for(counter = 0; counter < 512; counter++){
	wfs[counter] = sector_0[counter];}

	__asm{
		mov AL, 00h    							  // drive A:
		mov CX, 01h                           // number of sectors to write
		mov DX, 00h									  // boot sector
		LEA BX, WORD PTR wfs                  // ES:BX
		int 26h}                              // absolute write interrupt

// WRITE SECTOR 1
	for(counter = 0; counter < 512; counter++){
	wfs[counter] = sector_1_11[counter];}

	__asm{
		mov AL, 00h    							  // drive A:
		mov CX, 01h                           // number of sectors to write
		mov DX, 01h                           // sector 1
		LEA BX, WORD PTR wfs                  // ES:BX
		int 26h}                              // absolute write interrupt

 // WRITE SECTOR 11
	for(counter = 0; counter < 512; counter++){
	wfs[counter] = sector_1_11[counter];}

	__asm{
		mov AL, 00h    							  // drive A:
		mov CX, 01h                           // number of sectors to write
		mov DX, 0Bh                           // sector 11
		LEA BX, WORD PTR wfs                  // ES:BX
		int 26h}                              // absolute write interrupt

 // WRITE SECTOR 21
	for(counter = 0; counter < 512; counter++){
	wfs[counter] = sector_21[counter];}

	__asm{
		mov AL, 00h    							  // drive A:
		mov CX, 01h                           // number of sectors to write
		mov DX, 15h                           // sector 21
		LEA BX, WORD PTR wfs                  // ES:BX
		int 26h}                              // absolute write interrupt

// WRITE FAT SECTORS AGAIN USING INT 13h
	for(counter = 0; counter < 5; counter++){
	__asm{
		mov AH, 00h		// reset
		mov DL, 00h		// drive A:
		int 13h}}

	for(counter = 0; counter < 20; counter++){
	__asm{
		mov AH, 03h						// write function
		mov AL, 01h						// number of sectors to write
		mov CH, 00h						// track
		mov CL, 01h						// sector 1
		mov DH, 00h						// side
		mov DL, 00h						// drive A:
		mov BX, seg sector_0       // ES:BX
		mov ES, BX
		mov BX, offset sector_0
		int 13h}}

	for(counter = 0; counter < 20; counter++){
		  if(counter < 10) sector = 2;
		  if(counter >= 10) sector = 12;
	__asm{
		mov AH, 03h						// write function
		mov AL, 01h						// number of sectors to write
		mov CH, 00h						// track
		mov CL, BYTE PTR sector		// sector 2 or 12
		mov DH, 00h						// side
		mov DL, 00h						// drive A:
		mov BX, seg sector_1_11    // ES:BX
		mov ES, BX
		mov BX, offset sector_1_11
		int 13h}}

	for(counter = 0; counter < 10; counter++){
	__asm{
		mov AH, 03h		// write function
		mov AL, 01h		// number of sectors to write
		mov CH, 00h		// track
		mov CL, 01h		// sector 1
		mov DH, 01h		// side
		mov DL, 00h		// drive A:
		mov BX, seg sector_21
		mov ES, BX
		mov BX, offset sector_21
		int 13h}}


// RESTORE SCREEN
	position_cursor(0,0);
	set_color(0x000F);
   
// DISPLAY COPYWRONG
	printf("\n\n     Freehafer's DMF Port Floppy Format Program Is Terminated.\n     (c)2002 INNIEA PUBLISHING COMPANY  R%c%cK NOG!\n\n     TO VIEW THE NEW FORMAT YOU NEED TO RESTART THE\n     COMPUTER, OR VIEW THE DISK IN WINDOWS IN MOST CASES!\n\n",(char)153,(char)142);}
